import unittest
from decimal import Decimal

from httpretty import HTTPretty, httprettified

import openexchangerates

from datetime import date as Date

class TestOpenExchangeRates(unittest.TestCase):

    _FIXTURE_CURRENCIES = """{
    "AED": "United Arab Emirates Dirham",
    "AFN": "Afghan Afghani",
    "ALL": "Albanian Lek"
}
"""

    _FIXTURE_LATEST = """{
    "disclaimer": "<Disclaimer data>",
    "license": "<License data>",
    "timestamp": 1358150409,
    "base": "USD",
    "rates": {
        "AED": 3.666311,
        "AFN": 51.2281,
        "ALL": 104.748751
    }
}
"""

    _FIXTURE_HISTORICAL = """{
    "disclaimer": "<Disclaimer data>",
    "license": "<License data>",
    "timestamp": 1358150409,
    "base": "USD",
    "rates": {
        "AED": 3.666311,
        "AFN": 51.2281,
        "ALL": 104.748751
    }
}
"""

    @httprettified
    def test_historical(self):
        """Tests openexchangerates.OpenExchangeRateClient.historical``"""
        client = openexchangerates.OpenExchangeRatesClient('DUMMY_API_KEY')
        date = Date.fromtimestamp(1358150409)
        HTTPretty.register_uri(HTTPretty.GET, client.ENDPOINT_HISTORICAL %
                               date.strftime("%Y-%m-%d"),
                               body=self._FIXTURE_LATEST)
        historical = client.historical(date)
        self.assertIn('rates', historical)
        rates = historical['rates']
        self.assertEqual(len(rates), 3)
        self.assertIn('AED', rates)
        self.assertEqual(rates['AED'], Decimal('3.666311'))
        self.assertIn('AFN', rates)
        self.assertEqual(rates['AFN'], Decimal('51.2281'))
        self.assertIn('ALL', rates)
        self.assertEqual(rates['ALL'], Decimal('104.748751'))

    @httprettified
    def test_currencies(self):
        """Tests ``openexchangerates.OpenExchangeRateClient\.currencies``"""
        client = openexchangerates.OpenExchangeRatesClient('DUMMY_API_KEY')
        HTTPretty.register_uri(HTTPretty.GET, client.ENDPOINT_CURRENCIES,
                               body=self._FIXTURE_CURRENCIES)
        currencies = client.currencies()
        self.assertEqual(len(currencies), 3)
        self.assertIn('AED', currencies)
        self.assertIn('AFN', currencies)
        self.assertIn('ALL', currencies)

    @httprettified
    def test_latest(self):
        """Tests openexchangerates.OpenExchangeRateClient.latest``"""
        client = openexchangerates.OpenExchangeRatesClient('DUMMY_API_KEY')
        HTTPretty.register_uri(HTTPretty.GET, client.ENDPOINT_LATEST,
                               body=self._FIXTURE_LATEST)
        latest = client.latest()
        self.assertIn('rates', latest)
        rates = latest['rates']
        self.assertEqual(len(rates), 3)
        self.assertIn('AED', rates)
        self.assertEqual(rates['AED'], Decimal('3.666311'))
        self.assertIn('AFN', rates)
        self.assertEqual(rates['AFN'], Decimal('51.2281'))
        self.assertIn('ALL', rates)
        self.assertEqual(rates['ALL'], Decimal('104.748751'))

    @httprettified
    def test_exception(self):
        """Tests ``openexchangerates.OpenExchangeRateClientException``"""
        client = openexchangerates.OpenExchangeRatesClient('DUMMY_API_KEY')
        HTTPretty.register_uri(HTTPretty.GET, client.ENDPOINT_LATEST,
                               status=404)
        with(self.assertRaises(
                openexchangerates.OpenExchangeRatesClientException)) as e:
            client.latest()



import decimal

import requests

__version__ = '0.1.0'
__author__ = 'Metglobal'
__license__ = 'MIT'
__copyright__ = 'Copyright 2013 Metglobal'


class OpenExchangeRatesClientException(requests.exceptions.RequestException):
    """Base client exception wraps all kinds of ``requests`` lib exceptions"""
    pass


class OpenExchangeRatesClient(object):
    """This class is a client implementation for openexchangerate.org service

    """
    BASE_URL = 'http://openexchangerates.org/api'
    ENDPOINT_LATEST = BASE_URL + '/latest.json'
    ENDPOINT_CURRENCIES = BASE_URL + '/currencies.json'
    ENDPOINT_HISTORICAL = BASE_URL + '/historical/%s.json'

    def __init__(self, api_key):
        """Convenient constructor"""
        self.client = requests.Session()
        self.client.params.update({'app_id': api_key})

    def latest(self, base='USD'):
        """Fetches latest exchange rate data from service

        :Example Data:
            {
                disclaimer: "<Disclaimer data>",
                license: "<License data>",
                timestamp: 1358150409,
                base: "USD",
                rates: {
                    AED: 3.666311,
                    AFN: 51.2281,
                    ALL: 104.748751,
                    AMD: 406.919999,
                    ANG: 1.7831,
                    ...
                }
            }
        """
        try:
            resp = self.client.get(self.ENDPOINT_LATEST, params={'base': base})
            resp.raise_for_status()
        except requests.exceptions.RequestException as e:
            raise OpenExchangeRatesClientException(e)
        return resp.json(parse_int=decimal.Decimal,
                         parse_float=decimal.Decimal)

    def currencies(self):
        """Fetches current currency data of the service

        :Example Data:

        {
            AED: "United Arab Emirates Dirham",
            AFN: "Afghan Afghani",
            ALL: "Albanian Lek",
            AMD: "Armenian Dram",
            ANG: "Netherlands Antillean Guilder",
            AOA: "Angolan Kwanza",
            ARS: "Argentine Peso",
            AUD: "Australian Dollar",
            AWG: "Aruban Florin",
            AZN: "Azerbaijani Manat"
            ...
        }
        """
        try:
            resp = self.client.get(self.ENDPOINT_CURRENCIES)
        except requests.exceptions.RequestException as e:
            raise OpenExchangeRatesClientException(e)

        return resp.json()

    def historical(self, date, base='USD'):
        """Fetches historical exchange rate data from service

        :Example Data:
            {
                disclaimer: "<Disclaimer data>",
                license: "<License data>",
                timestamp: 1358150409,
                base: "USD",
                rates: {
                    AED: 3.666311,
                    AFN: 51.2281,
                    ALL: 104.748751,
                    AMD: 406.919999,
                    ANG: 1.7831,
                    ...
                }
            }
        """
        try:
            resp = self.client.get(self.ENDPOINT_HISTORICAL %
                                   date.strftime("%Y-%m-%d"),
                                   params={'base': base})
            resp.raise_for_status()
        except requests.exceptions.RequestException as e:
            raise OpenExchangeRatesClientException(e)
        return resp.json(parse_int=decimal.Decimal,
                         parse_float=decimal.Decimal)



from setuptools import setup

setup(
    name='openexchangerates3',
    version='0.1.1',
    description='openexchangerates.org Python3 API client',
    long_description=open('README.rst').read(),
    url='https://github.com/lihan/openexchangerates3',
    license='MIT',
    author='Lihan Li',
    author_email='llihan673@gmail.com',
    packages=['openexchangerates'],
    install_requires=[
        'requests',
    ],
    tests_require=[
        'httpretty',
        'mock'
    ],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet :: WWW/HTTP',
    ],
)
